@@ -22,12 +22,13 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, sevc1_new, interaction )
   USE fft_interfaces,       ONLY : fwfft
   USE gvecs,                ONLY : nls, nlsm
   USE gvect,                ONLY : nl, ngm, gstart, g, gg
-  USE fft_base,             ONLY : dfftp
+  USE fft_base,             ONLY : dfftp, dffts
   USE io_global,            ONLY : stdout
   USE kinds,                ONLY : dp
   USE klist,                ONLY : nks, xk
   USE lr_variables,         ONLY : evc0, revc0, rho_1, lr_verbosity,&
-                                 & ltammd, size_evc, no_hxc 
+                                 & ltammd, size_evc, no_hxc, lr_exx, &
+                                 & scissor, rho_1c,davidson
   USE realus,               ONLY : igk_k,npw_k
   USE lsda_mod,             ONLY : nspin
   USE uspp,                 ONLY : vkb, nkb, okvan
@@ -47,6 +48,14 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, sevc1_new, interaction )
   USE io_global,            ONLY : stdout
   USE dfunct,               ONLY : newq
   USE control_flags,        ONLY : tqr
+  USE mp,                   ONLY : mp_sum, mp_barrier
+  USE mp_global,            ONLY : intra_bgrp_comm
+  use lr_exx_kernel
+#ifdef __ENVIRON
+  USE plugin_flags,         ONLY : use_environ
+  USE scf,                  ONLY : rho
+  USE solvent_tddfpt,       ONLY : calc_vsolvent_tddfpt
+#endif
   !
   !
   IMPLICIT NONE
@@ -66,8 +75,15 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, sevc1_new, interaction )
   REAL(kind=dp), ALLOCATABLE :: dvrs(:,:), dvrss(:)
   REAL(kind=dp), ALLOCATABLE :: d_deeq(:,:,:,:)
   !
+  ! Environ related arrays
+  !
+  REAL(kind=dp), ALLOCATABLE :: &
+          dv_pol(:), &  ! response polarization potential
+          dv_epsilon(:) ! response dielectric potential
+  !
   COMPLEX(kind=dp), ALLOCATABLE :: dvrs_temp(:,:)   
   COMPLEX(kind=dp), ALLOCATABLE :: spsi1(:,:)
+  COMPLEX(kind=dp), ALLOCATABLE :: dvrsc(:,:), dvrssc(:)
   !
   COMPLEX(kind=dp) :: fp, fm
   !
@@ -93,10 +109,17 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, sevc1_new, interaction )
      !
      ! Calculate the full L
      !
-     ALLOCATE( dvrs(dfftp%nnr, nspin) )
-     ALLOCATE( dvrss(dffts%nnr) )
-     dvrs(:,:)=0.0d0
-     dvrss(:)=0.0d0
+     IF(gamma_only) THEN
+        ALLOCATE( dvrs(dfftp%nnr, nspin) )
+        ALLOCATE( dvrss(dffts%nnr) )
+        dvrs(:,:)=0.0d0
+        dvrss(:)=0.0d0
+     ELSE
+        ALLOCATE( dvrsc(dfftp%nnr, nspin) )
+        ALLOCATE( dvrssc(dffts%nnr) )
+        dvrsc(:,:)=0.0d0
+        dvrssc(:) =0.0d0
+     ENDIF
      !
      ! Calculation of the charge density response
      !
@@ -111,27 +134,65 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, sevc1_new, interaction )
         ! With no_hxc=.true. we recover the independent electrion 
         ! approximation, so we zero the interation.
         !
-        dvrs(:,1)=0.0d0
-        CALL interpolate (dvrs(:,1),dvrss,-1)
+        IF(gamma_only) THEN
+           dvrs(:,1)=0.0d0
+           CALL interpolate (dvrs(:,1),dvrss,-1)
+        ELSE
+           dvrsc(:,1)=0.0d0
+           CALL cinterpolate (dvrsc(:,1),dvrssc,-1)
+        ENDIF
         !
      ELSE
         !
-        dvrs(:,1)=rho_1(:,1)
-        !
-        ! In the gamma_only case dvrs is real, but dv_of_drho expects
-        ! a complex array on input, hence this temporary variable.
-        !
-        ALLOCATE( dvrs_temp(dfftp%nnr, nspin) )
-        dvrs_temp = CMPLX( dvrs, 0.0d0, kind=dp )         
-        !
-        DEALLOCATE ( dvrs )
-        !
-        CALL dv_of_drho(0,dvrs_temp,.FALSE.)
-        !
-        ALLOCATE ( dvrs(dfftp%nnr, nspin) ) 
-        dvrs=DBLE(dvrs_temp)
-        DEALLOCATE(dvrs_temp)
-        !
+        IF(gamma_only) THEN
+           dvrs(:,1)=rho_1(:,1)
+           !
+           ! In the gamma_only case dvrs is real, but dv_of_drho expects
+           ! a complex array on input, hence this temporary variable.
+           !
+           ALLOCATE( dvrs_temp(dfftp%nnr, nspin) )
+           dvrs_temp = CMPLX( dvrs, 0.0d0, kind=dp )         
+           !
+           DEALLOCATE ( dvrs )
+           !
+           CALL dv_of_drho(0,dvrs_temp,.FALSE.)
+           !
+           ALLOCATE ( dvrs(dfftp%nnr, nspin) ) 
+           dvrs=DBLE(dvrs_temp)
+           DEALLOCATE(dvrs_temp)
+           !
+#ifdef __ENVIRON
+           !
+           IF ( use_environ ) THEN
+              !
+              ALLOCATE( dv_pol(dfftp%nnr) )
+              ALLOCATE( dv_epsilon(dfftp%nnr) )
+              dv_pol(:) = 0.0d0
+              dv_epsilon(:) = 0.0d0
+              !
+              WRITE( stdout, '(5x,"ENVIRON: Calculate the response &
+                          & polarization and dielectric potentials")' )
+              !
+              CALL calc_vsolvent_tddfpt(dfftp%nnr, nspin, rho%of_r(:,1), &
+                                       & rho_1(:,1), dv_pol, dv_epsilon)
+              !
+              ! Add the response polarization and dielectric potentials
+              ! to the response HXC potential.
+              !
+              dvrs(:,1) = dvrs(:,1) + dv_pol(:) + dv_epsilon(:)
+              !
+              DEALLOCATE( dv_pol )
+              DEALLOCATE( dv_epsilon )
+              !
+           ENDIF
+#endif
+           !
+        ELSE
+           dvrsc(:,1)=rho_1c(:,1)
+           !
+           CALL dv_of_drho(0,dvrsc,.FALSE.)
+           !
+        ENDIF
         IF ( okvan )  THEN
            IF ( tqr ) THEN
               CALL newq_r(dvrs,d_deeq,.TRUE.)
@@ -148,14 +209,18 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, sevc1_new, interaction )
         !
         ! Put the nteraction on the smooth grid.
         !
-        CALL interpolate (dvrs(:,1),dvrss,-1)
+        IF (gamma_only) THEN
+           CALL interpolate (dvrs(:,1),dvrss,-1)
+        ELSE
+           CALL cinterpolate (dvrsc(:,1),dvrssc,-1)
+        ENDIF
      ENDIF
      !
   ENDIF
   !
   ! Make sure the psic workspace is availible
   !
-  ALLOCATE ( psic (dfftp%nnr) )
+  ALLOCATE ( psic (dffts%nnr) )
   !
   IF( gamma_only ) THEN
      !
@@ -169,11 +234,11 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, sevc1_new, interaction )
   !
   DEALLOCATE ( psic )
   !
-  IF ( interaction .and. (.not.ltammd) ) THEN
+  IF ( (interaction .or. lr_exx) .and. (.not.ltammd)  ) THEN
      !
      !   Normal interaction
      !
-     WRITE(stdout,'(5X,"lr_apply_liouvillian: &
+     if(.not. davidson) WRITE(stdout,'(5X,"lr_apply_liouvillian: &
           &applying interaction: normal")')
      !
      ! Here we add the two terms: 
@@ -187,7 +252,7 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, sevc1_new, interaction )
      !
      !   Tamm-dancoff interaction
      !
-     WRITE(stdout,'(5X,"lr_apply_liouvillian:&
+     if(.not. davidson) WRITE(stdout,'(5X,"lr_apply_liouvillian:&
           & applying interaction: tamm-dancoff")')
      !
      !   Here evc1_new contains the interaction
@@ -200,7 +265,7 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, sevc1_new, interaction )
      !
      !   Non interacting
      !
-     WRITE(stdout,'(5X,"lr_apply_liouvillian:&
+     if(.not. davidson) WRITE(stdout,'(5X,"lr_apply_liouvillian:&
           & not applying interaction")') 
      !
   ENDIF
@@ -251,10 +316,11 @@ SUBROUTINE lr_apply_liouvillian( evc1, evc1_new, sevc1_new, interaction )
   DEALLOCATE(spsi1)
   !
   !
-  CALL stop_clock('lr_apply')
   IF (interaction)      CALL stop_clock('lr_apply_int')
   IF (.not.interaction) CALL stop_clock('lr_apply_no')
   !
+  CALL stop_clock('lr_apply')
+  !
   RETURN
   !
 CONTAINS
@@ -266,11 +332,15 @@ CONTAINS
     USE realus,                   ONLY : tg_psic
     USE mp_global,                ONLY : me_bgrp
     USE fft_base,                 ONLY : dffts, tg_gather
+    USE mp_global,                ONLY : ibnd_start, ibnd_end, inter_bgrp_comm
+    USE mp,                       ONLY : mp_sum
+    USE lr_exx_kernel,            ONLY : lr_exx_sum_int
     !
     REAL(kind=dp), ALLOCATABLE :: becp2(:,:)
     REAL(kind=dp), ALLOCATABLE :: tg_dvrss(:)
     LOGICAL :: use_tg
     INTEGER :: v_siz, incr, ioff
+    INTEGER   :: ibnd_start_gamma, ibnd_end_gamma
     !
     use_tg=dffts%have_task_groups
     incr = 2
@@ -346,7 +416,14 @@ CONTAINS
        !
        evc1_new(:,:,:)=(0.0d0,0.0d0)
        !
-       DO ibnd=1,nbnd,incr
+       ibnd_start_gamma=ibnd_start
+       IF (MOD(ibnd_start, 2)==0) ibnd_start_gamma = ibnd_start+1
+       ibnd_end_gamma = MAX(ibnd_end, ibnd_start_gamma)
+       !
+       IF (lr_exx) CALL lr_exx_sum_int()
+       !
+       DO ibnd=ibnd_start_gamma,ibnd_end_gamma,incr
+!       DO ibnd=1,nbnd,2
           !
           ! Product with the potential vrs = (vltot+vr)
           ! revc0 is on smooth grid. psic is used up to smooth grid
@@ -369,6 +446,10 @@ CONTAINS
              !
           ENDIF
           !
+          IF (lr_exx) THEN
+             CALL lr_exx_apply_revc_int(psic, ibnd, nbnd,1)
+          ENDIF
+          !
           IF (real_space_debug > 7 .and. okvan .and. nkb > 0) THEN
           !THE REAL SPACE PART (modified from s_psi)
                   !fac = sqrt(omega)
@@ -433,10 +514,14 @@ CONTAINS
           !
           !   Back to reciprocal space 
           !
+
           CALL bfft_orbital_gamma (evc1_new(:,:,1), ibnd, nbnd,.false.)
           !
        ENDDO
        !
+#ifdef __MPI
+       CALL mp_sum( evc1_new(:,:,1), inter_bgrp_comm )
+#endif
        IF(dffts%have_task_groups) DEALLOCATE (tg_dvrss)
        !
        !
@@ -451,6 +536,8 @@ CONTAINS
        !
     ENDIF
     !
+    IF (lr_exx .AND. .NOT.interaction) CALL lr_exx_kernel_noint(evc1,evc1_new)
+    !
     !   Call h_psi on evc1 such that h.evc1 = sevc1_new
     !
     CALL h_psi(npwx,npw_k(1),nbnd,evc1(1,1,1),sevc1_new(1,1,1))
@@ -471,8 +558,8 @@ CONTAINS
     !
     DO ibnd=1,nbnd
        !
-       CALL zaxpy(npw_k(1), CMPLX(-et(ibnd,1),0.0d0,dp), &
-            &spsi1(:,ibnd), 1, sevc1_new(:,ibnd,1), 1)
+       CALL zaxpy(npw_k(1), CMPLX(-(et(ibnd,1)-scissor), &
+            &0.0d0,dp), spsi1(:,ibnd), 1, sevc1_new(:,ibnd,1), 1)
        !
     ENDDO
     !
@@ -485,6 +572,7 @@ CONTAINS
   SUBROUTINE lr_apply_liouvillian_k()
     !
     USE lr_variables,        ONLY : becp1_c
+    USE wvfct,               ONLY : current_k, npw
     !
     COMPLEX(kind=dp), ALLOCATABLE :: becp2(:,:)
     !
@@ -514,9 +602,14 @@ CONTAINS
              !
              DO ir=1,dffts%nnr
                 !
-                psic(ir)=revc0(ir,ibnd,ik)*cmplx(dvrss(ir),0.0d0,dp)
+                psic(ir)=revc0(ir,ibnd,ik)*dvrssc(ir)
                 !
              ENDDO
+!             IF (lr_exx) psic(:) = psic(:) + 0.5d0 * revc_int_c(:,ibnd,ik)
+             IF (lr_exx) THEN
+                CALL lr_exx_apply_revc_int(psic, ibnd, nbnd, ik)
+             ENDIF
+
              !
              !   Back to reciprocal space
              !
@@ -589,6 +682,7 @@ CONTAINS
     !   Call h_psi on evc1
     !   h_psi uses arrays igk and npw, so restore those
     !
+    IF (lr_exx .AND. .NOT.interaction) CALL lr_exx_kernel_noint(evc1,evc1_new)
     DO ik=1,nks
        !
        CALL init_us_2(npw_k(ik),igk_k(1,ik),xk(1,ik),vkb)
@@ -602,6 +696,8 @@ CONTAINS
        ENDDO
        !
        igk(:)=igk_k(:,ik)
+       current_k = ik
+       npw=npw_k(ik)
        !
        CALL h_psi(npwx,npw_k(ik),nbnd,evc1(1,1,ik),sevc1_new(1,1,ik))
        !
