@@ -27,6 +27,8 @@ MODULE lr_variables
   INTEGER :: nwordd0psi
   INTEGER :: nwordrestart
   !
+  CHARACTER (len=10), PARAMETER :: code = 'turboTDDFT'
+  !
   INTEGER :: n_ipol
   !
   !
@@ -34,6 +36,28 @@ MODULE lr_variables
   !
   CHARACTER (len=24) :: bgz_suffix
   !
+  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+  !Variables for TDHF using Dario Rocca's BSE routines.
+  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+  LOGICAL :: lr_exx
+
+  REAL(kind=dp), ALLOCATABLE :: xkk_m(:,:), xkk_p(:,:), weight_kk(:)
+  REAL(kind=dp) :: exxdiv
+  REAL(kind=dp) :: ecutfock_
+!  REAL(kind=dp), ALLOCATABLE :: revc_int(:,:)
+!  COMPLEX(kind=dp), ALLOCATABLE :: revc_int_c(:,:,:)
+
+  INTEGER, ALLOCATABLE :: &
+       igkk_kk_m(:,:),&
+       npw_kk_m(:),&
+       ind_kk_m(:,:),&
+       igkk_kk_p(:,:),&
+       npw_kk_p(:),&
+       ind_kk_p(:,:)
+  REAL(kind=dp) :: scissor
+  CHARACTER(len=200) :: eig_dir
+  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+  !
   !LOGICAL :: nlcc_any  ! .T. if any atom-type has nlcc
   !
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
@@ -64,7 +88,7 @@ MODULE lr_variables
                                ! index 1: q' using rotated SBR 2: p')
        evc1(:,:,:,:),     &    !  "    "
        evc1_new(:,:,:,:), &    !  "    "
-       sevc1_new(:,:,:,:),&    ! S * "    "
+       sevc1(:,:,:,:), sevc1_new(:,:,:,:),&    ! S * "    "
        d0psi(:,:,:,:)          ! for saving the original starting vectors
 
   !
@@ -72,11 +96,11 @@ MODULE lr_variables
   COMPLEX(kind=dp), ALLOCATABLE :: tg_revc0(:,:,:) !ground state wavefunctions in real space
   !
   real(kind=dp), ALLOCATABLE :: &
-       rho_1(:,:),           &         ! response charge density in real space
+       rho_1(:,:)                    ! response charge density in real space
        !rho_tot(:),        &         ! ground state + resp. charge density in real space (obm: is it used at all?)
-       rho_1_tot(:,:)                !response charge density (mode 2)
   COMPLEX(kind=dp), ALLOCATABLE :: &
-       rho_1_tot_im(:,:)             !response charge density, imaginary part used in resonance condition
+       rho_1c(:,:)      ! response charge density in real space
+
   !
   !integer, allocatable :: &
   !     igk_k(:,:),&
@@ -138,6 +162,8 @@ MODULE lr_variables
   real(kind=dp), ALLOCATABLE :: vl(:,:),vr(:,:)
   !
    REAL(kind=dp) :: norm0(3)
+
+   LOGICAL :: davidson = .false.           
   !
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   ! lr_input:
@@ -161,6 +187,8 @@ MODULE lr_variables
   INTEGER :: itermax            ! number of Lanczos vectors to be calculated
   INTEGER :: itermax_int        ! interpolated number of lanczos steps for Ritz vectors
   LOGICAL :: ltammd             ! Tarn-Darnkhoff approximation
+  LOGICAL :: d0psi_rs         ! Calculate d0psi in the real space
+  LOGICAL :: pseudo_hermitian   ! If psedudo hermitian algorithm is used
   LOGICAL :: no_hxc             ! If .true. no hartree exchange correlation corrections will be considered.
   LOGICAL :: project            ! If .true. projections to read virtual states will be calculated
   !
@@ -190,7 +218,7 @@ SUBROUTINE check_vector_gamma (x)
 ! component
 ! input, evc
 ! output : screen output
-   USE mp_global,            ONLY : inter_pool_comm, intra_pool_comm
+   USE mp_global,            ONLY : inter_pool_comm, intra_bgrp_comm
    USE mp,                   ONLY : mp_sum
    USE realus,               ONLY : npw_k
   USE gvect,                ONLY : gstart
@@ -206,7 +234,7 @@ IMPLICIT NONE
       temp_gamma = 2.D0*DDOT(2*npw_k(1),x(:),1,x(:),1)
        IF (gstart==2) temp_gamma = temp_gamma - dble(x(1))*dble(x(1))
 #ifdef __MPI
-       CALL mp_sum(temp_gamma, intra_pool_comm)
+       CALL mp_sum(temp_gamma, intra_bgrp_comm)
 #endif
       WRITE(stdout,'("<x> = ",E15.8)') temp_gamma
     END SUBROUTINE check_vector_gamma
@@ -217,7 +245,7 @@ SUBROUTINE check_vector_f (x)
 ! component
 ! input, evc
 ! output : screen output
-   USE mp_global,            ONLY : inter_pool_comm, intra_pool_comm
+   USE mp_global,            ONLY : inter_pool_comm, intra_bgrp_comm
    USE mp,                   ONLY : mp_sum
    USE realus,               ONLY : npw_k
   USE gvect,                ONLY : gstart
@@ -228,11 +256,11 @@ IMPLICIT NONE
   COMPLEX(kind=dp),INTENT(in)  :: x(:)
   !local
   COMPLEX(kind=dp) :: temp_f
-  COMPLEX(kind=dp), EXTERNAL    :: ZDOTC
+  COMPLEX(kind=dp), EXTERNAL    :: zdotc
 
-      temp_f = ZDOTC(npw_k(1),x(:),1,x(:),1)
+      temp_f = zdotc(npw_k(1),x(:),1,x(:),1)
 #ifdef __MPI
-       CALL mp_sum(temp_f, intra_pool_comm)
+       CALL mp_sum(temp_f, intra_bgrp_comm)
 #endif
       WRITE(stdout,'("<x> = ",2E15.8,1X)') temp_f
  END SUBROUTINE check_vector_f
@@ -241,7 +269,7 @@ SUBROUTINE check_all_bands_gamma (x,sx,nbnd1,nbnd2)
 ! Checks all bands of given KS states for orthoganilty
 ! input, evc and sevc
 ! output : screen output
-   USE mp_global,            ONLY : inter_pool_comm, intra_pool_comm
+   USE mp_global,            ONLY : inter_pool_comm, intra_bgrp_comm
    USE mp,                   ONLY : mp_sum
    USE realus,               ONLY : npw_k
    USE io_global,            ONLY : stdout
@@ -261,7 +289,7 @@ IMPLICIT NONE
       temp_gamma = 2.D0*DDOT(2*npw_k(1),x(:,ibnd),1,sx(:,jbnd),1)
        IF (gstart==2) temp_gamma = temp_gamma - dble(x(1,ibnd))*dble(sx(1,jbnd))
 #ifdef __MPI
-       CALL mp_sum(temp_gamma, intra_pool_comm)
+       CALL mp_sum(temp_gamma, intra_bgrp_comm)
 #endif
       WRITE(stdout,'("<x,",I02,"|S|x,",I02,"> =",E15.8)') ibnd,jbnd,temp_gamma
      ENDDO
@@ -272,7 +300,7 @@ SUBROUTINE check_density_gamma (rx,nbnd)
 ! Checks the contirbution of a given function transformed into real space
 ! input, revc
 ! output : screen output
-   USE mp_global,            ONLY : inter_pool_comm, intra_pool_comm
+   USE mp_global,            ONLY : inter_pool_comm, intra_bgrp_comm
    USE mp,                   ONLY : mp_sum
    USE realus,               ONLY : npw_k
    USE wvfct,                ONLY : wg
@@ -299,7 +327,7 @@ IMPLICIT NONE
        temp_gamma=sum(w1*dble(rx(1:dfftp%nnr,ibnd))*dble(rx(1:dfftp%nnr,ibnd))&
                +w2*aimag(rx(1:dfftp%nnr,ibnd))*aimag(rx(1:dfftp%nnr,ibnd)))
 #ifdef __MPI
-       CALL mp_sum(temp_gamma, intra_pool_comm)
+       CALL mp_sum(temp_gamma, intra_bgrp_comm)
 #endif
       WRITE(stdout,'("Contribution of bands ",I02," and ",I02," to total density",E15.8)') ibnd,ibnd+1,temp_gamma
     ENDDO
